@@ -19,6 +19,10 @@ package compressionFilters;
 import java.io.IOException;
 import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
+import java.util.Arrays;
+import java.util.HashMap;
+import java.util.Map;
+
 import javax.servlet.ServletOutputStream;
 import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpServletResponseWrapper;
@@ -29,18 +33,18 @@ import javax.servlet.http.HttpServletResponseWrapper;
  *
  * @author Amy Roh
  * @author Dmitri Valdin
- * @version $Id: CompressionServletResponseWrapper.java 939521 2010-04-30 00:16:33Z kkolinko $
  */
-
-public class CompressionServletResponseWrapper extends HttpServletResponseWrapper {
+public class CompressionServletResponseWrapper
+        extends HttpServletResponseWrapper {
 
     // ----------------------------------------------------- Constructor
 
     /**
      * Calls the parent constructor which creates a ServletResponse adaptor
      * wrapping the given response object.
+     *
+     * @param response The response object to be wrapped.
      */
-
     public CompressionServletResponseWrapper(HttpServletResponse response) {
         super(response);
         origResponse = response;
@@ -55,20 +59,12 @@ public class CompressionServletResponseWrapper extends HttpServletResponseWrappe
     /**
      * Original response
      */
-
-    protected HttpServletResponse origResponse = null;
-
-    /**
-     * Descriptive information about this Response implementation.
-     */
-
-    protected static final String info = "CompressionServletResponseWrapper";
+    protected final HttpServletResponse origResponse;
 
     /**
      * The ServletOutputStream that has been returned by
      * <code>getOutputStream()</code>, if any.
      */
-
     protected ServletOutputStream stream = null;
 
 
@@ -76,52 +72,79 @@ public class CompressionServletResponseWrapper extends HttpServletResponseWrappe
      * The PrintWriter that has been returned by
      * <code>getWriter()</code>, if any.
      */
-
     protected PrintWriter writer = null;
 
     /**
      * The threshold number to compress
      */
-    protected int threshold = 0;
+    protected int compressionThreshold = 0;
+
+    /**
+     * The compression buffer size
+     */
+    protected int compressionBuffer = 8192;  // 8KB default
+
+    /**
+     * The mime types to compress
+     */
+    protected String[] compressionMimeTypes = {"text/html", "text/xml", "text/plain"};
 
     /**
      * Debug level
      */
-    private int debug = 0;
+    protected int debug = 0;
 
     /**
-     * Content type
+     * keeps a copy of all headers set
      */
-    protected String contentType = null;
+    private final Map<String,String> headerCopies = new HashMap<>();
+
 
     // --------------------------------------------------------- Public Methods
 
 
     /**
-     * Set content type
-     */
-    public void setContentType(String contentType) {
-        if (debug > 1) {
-            System.out.println("setContentType to "+contentType);
-        }
-        this.contentType = contentType;
-        origResponse.setContentType(contentType);
-    }
-
-
-    /**
-     * Set threshold number
+     * Set threshold number.
+     *
+     * @param threshold The new compression threshold
      */
     public void setCompressionThreshold(int threshold) {
         if (debug > 1) {
             System.out.println("setCompressionThreshold to " + threshold);
         }
-        this.threshold = threshold;
+        this.compressionThreshold = threshold;
     }
 
+    /**
+     * Set compression buffer.
+     *
+     * @param buffer New size of buffer to use for compressed output
+     */
+    public void setCompressionBuffer(int buffer) {
+        if (debug > 1) {
+            System.out.println("setCompressionBuffer to " + buffer);
+        }
+        this.compressionBuffer = buffer;
+    }
 
     /**
-     * Set debug level
+     * Set compressible mime types.
+     *
+     * @param mimeTypes The new list of mime types that will be considered for
+     *                  compression
+     */
+    public void setCompressionMimeTypes(String[] mimeTypes) {
+        if (debug > 1) {
+            System.out.println("setCompressionMimeTypes to " +
+                    Arrays.toString(mimeTypes));
+        }
+        this.compressionMimeTypes = mimeTypes;
+    }
+
+    /**
+     * Set debug level.
+     *
+     * @param debug The new debug level
      */
     public void setDebugLevel(int debug) {
         this.debug = debug;
@@ -133,18 +156,23 @@ public class CompressionServletResponseWrapper extends HttpServletResponseWrappe
      * associated with this Response.
      *
      * @exception IOException if an input/output error occurs
+     *
+     * @return A new servlet output stream that compressed any data written to
+     *         it
      */
-    public ServletOutputStream createOutputStream() throws IOException {
+    protected ServletOutputStream createOutputStream() throws IOException {
         if (debug > 1) {
             System.out.println("createOutputStream gets called");
         }
 
-        CompressionResponseStream stream = new CompressionResponseStream(origResponse);
+        CompressionResponseStream stream = new CompressionResponseStream(
+                this, origResponse.getOutputStream());
         stream.setDebugLevel(debug);
-        stream.setBuffer(threshold);
+        stream.setCompressionThreshold(compressionThreshold);
+        stream.setCompressionBuffer(compressionBuffer);
+        stream.setCompressionMimeTypes(compressionMimeTypes);
 
         return stream;
-
     }
 
 
@@ -156,10 +184,12 @@ public class CompressionServletResponseWrapper extends HttpServletResponseWrappe
             if (writer != null) {
                 writer.close();
             } else {
-                if (stream != null)
+                if (stream != null) {
                     stream.close();
+                }
             }
         } catch (IOException e) {
+            // Ignore
         }
     }
 
@@ -172,9 +202,10 @@ public class CompressionServletResponseWrapper extends HttpServletResponseWrappe
      *
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public void flushBuffer() throws IOException {
         if (debug > 1) {
-            System.out.println("flush buffer @ CompressionServletResponseWrapper");
+            System.out.println("flush buffer @ GZipServletResponseWrapper");
         }
         ((CompressionResponseStream)stream).flush();
 
@@ -187,19 +218,21 @@ public class CompressionServletResponseWrapper extends HttpServletResponseWrappe
      *  already been called for this response
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public ServletOutputStream getOutputStream() throws IOException {
 
-        if (writer != null)
+        if (writer != null) {
             throw new IllegalStateException("getWriter() has already been called for this response");
+        }
 
-        if (stream == null)
+        if (stream == null) {
             stream = createOutputStream();
+        }
         if (debug > 1) {
             System.out.println("stream is set to "+stream+" in getOutputStream");
         }
 
-        return (stream);
-
+        return stream;
     }
 
     /**
@@ -209,37 +242,54 @@ public class CompressionServletResponseWrapper extends HttpServletResponseWrappe
      *  already been called for this response
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public PrintWriter getWriter() throws IOException {
 
-        if (writer != null)
-            return (writer);
+        if (writer != null) {
+            return writer;
+        }
 
-        if (stream != null)
+        if (stream != null) {
             throw new IllegalStateException("getOutputStream() has already been called for this response");
+        }
 
         stream = createOutputStream();
         if (debug > 1) {
             System.out.println("stream is set to "+stream+" in getWriter");
         }
-        //String charset = getCharsetFromContentType(contentType);
         String charEnc = origResponse.getCharacterEncoding();
         if (debug > 1) {
             System.out.println("character encoding is " + charEnc);
         }
-        // HttpServletResponse.getCharacterEncoding() shouldn't return null
-        // according the spec, so feel free to remove that "if"
-        if (charEnc != null) {
-            writer = new PrintWriter(new OutputStreamWriter(stream, charEnc));
+        writer = new PrintWriter(new OutputStreamWriter(stream, charEnc));
+
+        return writer;
+    }
+
+    @Override
+    public String getHeader(String name) {
+        return headerCopies.get(name);
+    }
+
+    @Override
+    public void addHeader(String name, String value) {
+        if (headerCopies.containsKey(name)) {
+            String existingValue = headerCopies.get(name);
+            if ((existingValue != null) && (existingValue.length() > 0)) {
+                headerCopies.put(name, existingValue + "," + value);
+            } else {
+                headerCopies.put(name, value);
+            }
         } else {
-            writer = new PrintWriter(stream);
+            headerCopies.put(name, value);
         }
-        
-        return (writer);
-
+        super.addHeader(name, value);
     }
 
 
-    public void setContentLength(int length) {
+    @Override
+    public void setHeader(String name, String value) {
+        headerCopies.put(name, value);
+        super.setHeader(name, value);
     }
-
 }
